@@ -36,7 +36,7 @@ SELECT n.nspname, t.typname, t.typlen, t.typbyval, t.typtype,
        t.typrelid, t.typelem, t.typinput, t.typoutput, t.typreceive,
        t.typsend, t.typmodin, t.typmodout, t.typanalyze, t.typalign,
        t.typstorage, t.typnotnull, t.typbasetype, t.typtypmod, t.typndims,
-       t.typcollation, t.typdefaultbin, t.typdefault
+       t.typcollation, t.typdefaultbin, t.typdefault, t.typacl
   FROM pg_type t, pg_namespace n
  WHERE t.typnamespace = n.oid
    AND n.nspname = 'dbms_stats'
@@ -53,17 +53,21 @@ UPDATE pg_statistic SET
     stakind2 = 1,
     stakind3 = 2,
     stakind4 = 3,
+    stakind5 = 5,
     staop1 = 14,
     staop2 = 11,
     staop3 = 12,
     staop4 = 13,
+    staop5 = 15,
     stanumbers1 = ARRAY[staattnum,4],
     stanumbers2 = ARRAY[staattnum,1],
     stanumbers3 = ARRAY[staattnum,2],
     stanumbers4 = ARRAY[staattnum,3],
+    stanumbers5 = ARRAY[staattnum,5],
     stavalues2 = array_cat(stavalues1,stavalues1),
     stavalues3 = array_cat(array_cat(stavalues1,stavalues1),stavalues1),
     stavalues4 = array_cat(array_cat(array_cat(stavalues1,stavalues1),stavalues1),stavalues1)
+   ,stavalues5 = array_cat(array_cat(array_cat(array_cat(stavalues1,stavalues1),stavalues1),stavalues1),stavalues1)
  WHERE starelid = 'st0'::regclass;
 SELECT dbms_stats.lock_table_stats('st0');
 UPDATE dbms_stats.column_stats_locked SET
@@ -75,18 +79,22 @@ UPDATE dbms_stats.column_stats_locked SET
     stakind2 = 3,
     stakind3 = 4,
     stakind4 = 1,
+    stakind5 = 5,
     staop1 = 22,
     staop2 = 23,
     staop3 = 24,
     staop4 = 21,
+    staop5 = 25,
     stanumbers1 = ARRAY[-staattnum,22],
     stanumbers2 = ARRAY[-staattnum,23],
     stanumbers3 = ARRAY[-staattnum,24],
     stanumbers4 = ARRAY[-staattnum,21],
+    stanumbers5 = ARRAY[-staattnum,25],
     stavalues1 = stavalues3,
     stavalues2 = stavalues2,
     stavalues3 = stavalues1,
     stavalues4 = stavalues4
+   ,stavalues5 = stavalues5
 ;
 
 /*
@@ -117,18 +125,22 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge(NULL, s)
          FROM pg_statistic s
         WHERE starelid = 'st0'::regclass
@@ -145,18 +157,22 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge(v, NULL)
          FROM dbms_stats.column_stats_locked v
         WHERE starelid = 'st0'::regclass
@@ -176,18 +192,22 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge(v, s)
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -207,18 +227,22 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge(v, s)
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -241,9 +265,13 @@ SELECT dbms_stats.merge(NULL, (
        s.starelid::regclass, s.staattnum, s.stainherit,
        s.stanullfrac, s.stawidth, s.stadistinct,
        s.stakind1, s.stakind2, s.stakind3, s.stakind4,
+       s.stakind5,
        s.staop1, s.staop2, s.staop3,
+       s.staop4,
        NULL, s.stanumbers1, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+       s.stanumbers5,
        s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+      ,s.stavalues5
        ))
   FROM pg_statistic s
  WHERE s.starelid = 'st0'::regclass
@@ -260,25 +288,33 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge(NULL, (
               s.starelid::regclass, s.staattnum, s.stainherit,
               s.stanullfrac, s.stawidth, s.stadistinct,
               s.stakind1, s.stakind2, s.stakind3, s.stakind4,
+              s.stakind5,
               s.staop1, s.staop2, s.staop3, s.staop4,
+              s.staop5,
               NULL, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+              s.stanumbers5,
               s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+             ,s.stavalues5
               ))
          FROM pg_statistic s
         WHERE s.starelid = 'st0'::regclass
@@ -289,9 +325,13 @@ SELECT dbms_stats.merge((
        v.starelid::regclass, v.staattnum, v.stainherit,
        v.stanullfrac, v.stawidth, v.stadistinct,
        v.stakind1, v.stakind2, v.stakind3, v.stakind4,
+       v.stakind5,
        v.staop1, v.staop2, v.staop3,
+       v.staop4,
        NULL, v.stanumbers1, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+       v.stanumbers5,
        v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+      ,v.stavalues5
        ), NULL)
   FROM dbms_stats.column_stats_locked v
  WHERE v.starelid = 'st0'::regclass
@@ -308,25 +348,33 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge((
               v.starelid::regclass, v.staattnum, v.stainherit,
               v.stanullfrac, v.stawidth, v.stadistinct,
               v.stakind1, v.stakind2, v.stakind3, v.stakind4,
+              v.stakind5,
               v.staop1, v.staop2, v.staop3, v.staop4,
+              v.staop5,
               NULL, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+              v.stanumbers5,
               v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+             ,v.stavalues5
               ), NULL)
          FROM dbms_stats.column_stats_locked v
         WHERE v.starelid = 'st0'::regclass
@@ -337,16 +385,24 @@ SELECT dbms_stats.merge((
        v.starelid::regclass, v.staattnum, v.stainherit,
        v.stanullfrac, v.stawidth, v.stadistinct,
        v.stakind1, v.stakind2, v.stakind3, v.stakind4,
+       v.stakind5,
        v.staop1, v.staop2, v.staop3,
+       v.staop4,
        NULL, v.stanumbers1, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+       v.stanumbers5,
        v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+      ,v.stavalues5
        ), (
        s.starelid::regclass, s.staattnum, s.stainherit,
        s.stanullfrac, s.stawidth, s.stadistinct,
        s.stakind1, s.stakind2, s.stakind3, s.stakind4,
+       s.stakind5,
        s.staop1, s.staop2, s.staop3,
+       s.staop4,
        NULL, s.stanumbers1, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+       s.stanumbers5,
        s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+      ,s.stavalues5
        ))
   FROM dbms_stats.column_stats_locked v,
        pg_statistic s
@@ -366,32 +422,44 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge((
               v.starelid::regclass, v.staattnum, v.stainherit,
               v.stanullfrac, v.stawidth, v.stadistinct,
               v.stakind1, v.stakind2, v.stakind3, v.stakind4,
+              v.stakind5,
               v.staop1, v.staop2, v.staop3, v.staop4,
+              v.staop5,
               NULL, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+              v.stanumbers5,
               v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+             ,v.stavalues5
               ), (
               s.starelid::regclass, s.staattnum, s.stainherit,
               s.stanullfrac, s.stawidth, s.stadistinct,
               s.stakind1, s.stakind2, s.stakind3, s.stakind4,
+              s.stakind5,
               s.staop1, s.staop2, s.staop3, s.staop4,
+              s.staop5,
               NULL, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+              s.stanumbers5,
               s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+             ,s.stavalues5
               ))
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -411,18 +479,22 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge((
              NULL, NULL, NULL,
              NULL, NULL, NULL,
@@ -430,6 +502,7 @@ SELECT (m.merge).starelid::regclass,
              NULL, NULL, NULL, NULL,
              NULL, NULL, NULL, NULL,
              NULL, NULL, NULL, NULL
+            ,NULL, NULL, NULL, NULL
              ), s)
          FROM pg_statistic s
         WHERE s.starelid = 'st0'::regclass
@@ -446,24 +519,29 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge(v, (
              NULL, NULL, NULL,
              NULL, NULL, NULL,
              NULL, NULL, NULL, NULL,
              NULL, NULL, NULL, NULL,
              NULL, NULL, NULL, NULL,
+             NULL, NULL, NULL, NULL,
              NULL, NULL, NULL, NULL))
          FROM dbms_stats.column_stats_locked v
         WHERE v.starelid = 'st0'::regclass
@@ -480,18 +558,22 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge(v, s)
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -508,12 +590,14 @@ SELECT dbms_stats.merge((
        NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL
+      ,NULL, NULL, NULL, NULL
        ), (
        s.starelid::regclass, s.staattnum, s.stainherit,
        s.stanullfrac, s.stawidth, s.stadistinct,
        NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL,
+       NULL, NULL, NULL, NULL,
        NULL, NULL, NULL, NULL))
   FROM dbms_stats.column_stats_locked v,
        pg_statistic s
@@ -533,32 +617,44 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge((
               v.starelid::regclass, v.staattnum, v.stainherit,
               v.stanullfrac, v.stawidth, v.stadistinct,
               v.stakind1, v.stakind2, v.stakind3, v.stakind4,
+              v.stakind5,
               v.staop1, v.staop2, v.staop3,
+              v.staop4,
               NULL, v.stanumbers1, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+              v.stanumbers5,
               v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+             ,v.stavalues5
               ), (
               s.starelid::regclass, s.staattnum, s.stainherit,
               s.stanullfrac, s.stawidth, s.stadistinct,
               s.stakind1, s.stakind2, s.stakind3, s.stakind4,
+              s.stakind5,
               s.staop1, s.staop2, s.staop3, s.staop4,
+              s.staop5,
               s.stanumbers1, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+              s.stanumbers5,
               s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+             ,s.stavalues5
               ))
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -572,16 +668,24 @@ SELECT dbms_stats.merge((
        v.starelid::regclass, v.staattnum, v.stainherit,
        v.stanullfrac, v.stawidth, v.stadistinct,
        v.stakind1, v.stakind2, v.stakind3, v.stakind4,
+       v.stakind5,
        v.staop1, v.staop2, v.staop3,
+       v.staop4,
        NULL, v.stanumbers1, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+       v.stanumbers5,
        v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+      ,v.stavalues5
        ), (
        s.starelid::regclass, s.staattnum, s.stainherit,
        s.stanullfrac, s.stawidth, s.stadistinct,
        s.stakind1, s.stakind2, s.stakind3, s.stakind4,
+       s.stakind5,
        s.staop1, s.staop2, s.staop3,
+       s.staop4,
        NULL, s.stanumbers1, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+       s.stanumbers5,
        s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+      ,s.stavalues5
        ))
   FROM dbms_stats.column_stats_locked v,
        pg_statistic s
@@ -601,32 +705,44 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge((
               v.starelid::regclass, v.staattnum, v.stainherit,
               v.stanullfrac, v.stawidth, v.stadistinct,
               '1', '1', '1', '1',
+              '1',
               v.staop1, v.staop2, v.staop3, v.staop4,
+              v.staop5,
               v.stanumbers1, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+              v.stanumbers5,
               v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+             ,v.stavalues5
               ), (
               s.starelid::regclass, s.staattnum, s.stainherit,
               s.stanullfrac, s.stawidth, s.stadistinct,
               '1', '1', '1', '1',
+              '1',
               s.staop1, s.staop2, s.staop3, s.staop4,
+              s.staop5,
               s.stanumbers1, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+              s.stanumbers5,
               s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+             ,s.stavalues5
               ))
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -646,31 +762,43 @@ SELECT (m.merge).starelid::regclass,
        (m.merge).stakind2,
        (m.merge).stakind3,
        (m.merge).stakind4,
+       (m.merge).stakind5,
        (m.merge).staop1,
        (m.merge).staop2,
        (m.merge).staop3,
        (m.merge).staop4,
+       (m.merge).staop5,
        (m.merge).stanumbers1,
        (m.merge).stanumbers2,
        (m.merge).stanumbers3,
        (m.merge).stanumbers4,
+       (m.merge).stanumbers5,
        (m.merge).stavalues1,
        (m.merge).stavalues2,
        (m.merge).stavalues3,
        (m.merge).stavalues4
+      ,(m.merge).stavalues5
  FROM (SELECT dbms_stats.merge((v.starelid::regclass, v.staattnum, v.stainherit,
               v.stanullfrac, v.stawidth, v.stadistinct,
               '2', '2', '2', '2',
+              '2',
               v.staop1, v.staop2, v.staop3, v.staop4,
+              v.staop5,
               v.stanumbers1, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+              v.stanumbers5,
               v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+             ,v.stavalues5
               ), (
               s.starelid::regclass, s.staattnum, s.stainherit,
               s.stanullfrac, s.stawidth, s.stadistinct,
               '2', '2', '2', '2',
+              '2',
               s.staop1, s.staop2, s.staop3, s.staop4,
+              s.staop5,
               s.stanumbers1, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+              s.stanumbers5,
               s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+             ,s.stavalues5
               ))
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -683,16 +811,24 @@ SELECT (m.merge).starelid::regclass,
 SELECT dbms_stats.merge((v.starelid::regclass, '2', v.stainherit,
               v.stanullfrac, v.stawidth, v.stadistinct,
               '1', '1', '1', '1',
+              '1',
               v.staop1, v.staop2, v.staop3, v.staop4,
+              v.staop5,
               v.stanumbers1, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+              v.stanumbers5,
               v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+             ,v.stavalues5
               ), (
               s.starelid::regclass, s.staattnum, s.stainherit,
               s.stanullfrac, s.stawidth, s.stadistinct,
               '1', '1', '1', '1',
+              '1',
               s.staop1, s.staop2, s.staop3, s.staop4,
+              s.staop5,
               s.stanumbers1, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+              s.stanumbers5,
               s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+             ,s.stavalues5
               ))
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -705,16 +841,24 @@ SELECT dbms_stats.merge((v.starelid::regclass, '2', v.stainherit,
 SELECT dbms_stats.merge((v.starelid::regclass, '2', v.stainherit,
               v.stanullfrac, v.stawidth, v.stadistinct,
               '2', '2', '2', '2',
+              '2',
               v.staop1, v.staop2, v.staop3, v.staop4,
+              v.staop5,
               v.stanumbers1, v.stanumbers2, v.stanumbers3, v.stanumbers4,
+              v.stanumbers5,
               v.stavalues1, v.stavalues2, v.stavalues3, v.stavalues4
+             ,v.stavalues5
               ), (
               s.starelid::regclass, s.staattnum, s.stainherit,
               s.stanullfrac, s.stawidth, s.stadistinct,
               '2', '2', '2', '2',
+              '2',
               s.staop1, s.staop2, s.staop3, s.staop4,
+              s.staop5,
               s.stanumbers1, s.stanumbers2, s.stanumbers3, s.stanumbers4,
+              s.stanumbers5,
               s.stavalues1, s.stavalues2, s.stavalues3, s.stavalues4
+             ,s.stavalues5
               ))
          FROM dbms_stats.column_stats_locked v,
               pg_statistic s
@@ -744,7 +888,7 @@ SELECT count(*) FROM dbms_stats.relation_stats_backup;
 SELECT count(*) FROM dbms_stats.column_stats_backup;
 
 -- No.7-1-10
--- #### 9.1 doesn't has materialized views
+-- #### 9.2 doesn't has materialized views
 -- DELETE FROM dbms_stats.relation_stats_backup;
 -- SELECT dbms_stats.backup(1, 's0.smv0'::regclass, NULL);
 -- SELECT count(*) FROM dbms_stats.relation_stats_backup;
@@ -772,7 +916,7 @@ SELECT starelid::regclass, staattnum FROM dbms_stats.column_stats_backup
 
 -- No.7-1-18
 DELETE FROM dbms_stats.relation_stats_backup;
-SELECT dbms_stats.backup(NULL, 's0.st0'::regclass, NULL);
+\! psql contrib_regression -c "SELECT dbms_stats.backup(NULL, 's0.st0'::regclass, NULL)" > results/ut_no2_1_17.out 2>&1
 SELECT count(*) FROM dbms_stats.relation_stats_backup;
 SELECT count(*) FROM dbms_stats.column_stats_backup;
 
@@ -861,7 +1005,7 @@ SELECT dbms_stats.backup('s0.sft0'::regclass, NULL, 'dummy comment');
 SELECT id, unit, comment FROM dbms_stats.backup_history;
 
 -- No.8-1-12
--- #### 9.1 doesn't has materialized views
+-- #### 9.2 doesn't has materialized views
 -- DELETE FROM dbms_stats.backup_history;
 -- SELECT dbms_stats.backup('s0.smv0'::regclass, NULL, 'dummy comment');
 -- SELECT id, unit, comment FROM dbms_stats.backup_history;
@@ -1137,10 +1281,12 @@ SELECT starelid, attname, stainherit FROM columns_locked_v c;
 DELETE FROM dbms_stats.relation_stats_locked;
 INSERT INTO dbms_stats.relation_stats_backup(
                id, relid, relname, relpages, reltuples,
+               relallvisible,
                curpages)
      VALUES (2,
              'pg_toast.pg_toast_2618'::regclass,
              'pg_toast.pg_toast_2618', 1, 1,
+             1,
              1);
 SELECT * FROM relations_backup_v
  WHERE id = 2
@@ -1164,8 +1310,10 @@ SELECT starelid, attname, stainherit FROM columns_locked_v c;
 DELETE FROM dbms_stats.relation_stats_locked;
 INSERT INTO dbms_stats.relation_stats_backup(
                id, relid, relname, relpages, reltuples,
+               relallvisible,
                curpages)
      VALUES (2, 's0.ss0'::regclass, 's0.ss0', 1, 1,
+             1,
              1);
 SELECT * FROM relations_backup_v
  WHERE id = 2
@@ -1181,8 +1329,10 @@ DELETE FROM dbms_stats.relation_stats_backup
 DELETE FROM dbms_stats.relation_stats_locked;
 INSERT INTO dbms_stats.relation_stats_backup(
                id, relid, relname, relpages, reltuples,
+               relallvisible,
                curpages)
      VALUES (2, 's0.sc0'::regclass, 's0.sc0', 1, 1,
+             1,
              1);
 SELECT * FROM relations_backup_v
  WHERE id = 2
@@ -1198,21 +1348,23 @@ DELETE FROM dbms_stats.relation_stats_backup
 DELETE FROM dbms_stats.relation_stats_locked;
 INSERT INTO dbms_stats.relation_stats_backup(
                id, relid, relname, relpages, reltuples,
+               relallvisible,
                curpages)
-     VALUES (2, 's0.sft0'::regclass, 's0.sft0', 1, 1,
+     VALUES (3, 's0.sft0'::regclass, 's0.sft0', 1, 1,
+             1,
              1);
 SELECT * FROM relations_backup_v
- WHERE id = 2
+ WHERE id = 3
    AND relname = 's0.sft0';
 SELECT dbms_stats.restore(2, 's0.sft0', NULL);
 SELECT count(*) FROM dbms_stats.column_stats_locked;
 SELECT count(*) FROM dbms_stats.relation_stats_locked;
 DELETE FROM dbms_stats.relation_stats_backup
- WHERE id = 2
+ WHERE id = 3
    AND relname = 's0.sft0';
 
 -- No.9-1-22
--- #### 9.1 doesn't has materialized views
+-- #### 9.2 doesn't has materialized views
 -- DELETE FROM dbms_stats.relation_stats_locked;
 -- INSERT INTO dbms_stats.relation_stats_backup(
 --                id, relid, relname, relpages, reltuples,
@@ -1235,8 +1387,10 @@ DELETE FROM dbms_stats.relation_stats_backup
 DELETE FROM dbms_stats.relation_stats_locked;
 INSERT INTO dbms_stats.relation_stats_backup(
                id, relid, relname, relpages, reltuples,
+               relallvisible,
                curpages)
      VALUES (2, 'pg_catalog.pg_class'::regclass, 'pg_catalog.pg_class', 1, 1,
+             1,
              1);
 SELECT * FROM relations_backup_v
  WHERE id = 2
@@ -1443,7 +1597,7 @@ SELECT dbms_stats.lock('s0.sft0', 'id');
 SELECT * FROM relations_locked_v;
 SELECT * FROM columns_locked_v c;
 -- No.11-1-13
--- #### 9.1 doesn't has materialized views
+-- #### 9.2 doesn't has materialized views
 -- DELETE FROM dbms_stats.relation_stats_locked;
 -- SELECT dbms_stats.lock('s0.smv0', 'id');
 -- SELECT * FROM relations_locked_v;
@@ -1464,21 +1618,28 @@ VACUUM ANALYZE;
 DELETE FROM dbms_stats.relation_stats_locked;
 INSERT INTO dbms_stats.relation_stats_locked(
     relid, relname, relpages, reltuples,
+    relallvisible,
     curpages)
     VALUES('s0.st0'::regclass, 's0.st0', 1, 1640,
+           1,
            1);
 SELECT dbms_stats.lock_column_stats('s0.st0','id');
 UPDATE dbms_stats.column_stats_locked
    SET (stanullfrac, stawidth, stadistinct,
         stakind1, stakind2, stakind3, stakind4,
+        stakind5,
         staop1, staop2, staop3, staop4,
+        staop5,
         stanumbers1, stanumbers2, stanumbers3, stanumbers4,
+        stanumbers5,
         stavalues1, stavalues2, stavalues3, stavalues4
+       ,stavalues5
        ) = (
         NULL, NULL, NULL,
         NULL, NULL, NULL, NULL,
         NULL, NULL, NULL, NULL,
         NULL, NULL, NULL, NULL,
+        NULL, NULL, NULL, NULL,
         NULL, NULL, NULL, NULL)
  WHERE starelid = 's0.st0'::regclass;
 SELECT dbms_stats.lock('s0.st0', 'id');
@@ -1536,7 +1697,7 @@ SELECT dbms_stats.lock('s0.sft0');
 SELECT * FROM relations_locked_v;
 SELECT * FROM columns_locked_v c;
 -- No.11-2-10
--- #### 9.1 doesn't has materialized views
+-- #### 9.2 doesn't has materialized views
 -- DELETE FROM dbms_stats.relation_stats_locked;
 -- SELECT dbms_stats.lock('s0.smv0');
 -- SELECT * FROM relations_locked_v;
@@ -1549,8 +1710,10 @@ DELETE FROM dbms_stats.relation_stats_locked;
 SELECT dbms_stats.lock_table_stats('s0.st0');
 UPDATE dbms_stats.relation_stats_locked
    SET (relpages, reltuples,
+        relallvisible,
         curpages)
      = (NULL, NULL, NULL
+       ,NULL
        )
  WHERE relid = 's0.st0'::regclass;
 SELECT dbms_stats.lock('s0.st0');
@@ -2575,3 +2738,66 @@ SELECT starelid, attname, stainherit FROM columns_locked_v c;
 SELECT relid::regclass FROM dbms_stats.relation_stats_locked
  GROUP BY relid
  ORDER BY relid;
+
+-- No.15 Make sure that the stats given by pg_dbms_stats doesn't
+-- ignored by ACL check
+CREATE FUNCTION testfunc() RETURNS TEXT AS $$
+DECLARE
+  ret text;
+  v text;
+BEGIN
+  ret = '';
+  FOR v IN EXPLAIN SELECT * FROM s0.st4 WHERE a < '000' LOOP
+    -- mask unnecessary values
+    v = regexp_replace(v, '(cost|width)=[0-9.]+', E'\\1=xxx', 'g');
+    ret = ret || v || E'\n';
+  END LOOP;    
+  RETURN ret;
+END;
+$$ LANGUAGE plpgsql;
+
+SET pg_dbms_stats.use_locked_stats TO on;
+CREATE TABLE s0.st4 (a text);
+INSERT INTO s0.st4 SELECT '1' || md5(g::text) FROM generate_series(1, 10000) as g;
+VACUUM ANALYZE s0.st4;
+-- should estimate that rows = 1, not 5000
+SELECT testfunc();
+SET pg_dbms_stats.use_locked_stats TO off;
+SELECT dbms_stats.lock_table_stats('s0.st4');
+SET pg_dbms_stats.use_locked_stats TO on;
+-- should estimate that rows = 1, not 5000
+SELECT testfunc();
+DROP TABLE s0.st4;
+SELECT dbms_stats.clean_up_stats();
+
+/*
+ * No.15-2 Ditto for index stats
+ */
+CREATE TABLE s0.st4 (a double precision);
+CREATE INDEX on s0.st4 (floor(log(a)));
+SELECT dbms_stats.lock_table_stats('s0.st4');
+INSERT INTO s0.st4 (SELECT a from GENERATE_SERIES(1, 99999) a);
+ANALYZE t1;
+SET pg_dbms_stats.use_locked_stats TO off;
+SELECT testfunc();
+SET pg_dbms_stats.use_locked_stats TO on;
+SELECT testfunc();
+DROP TABLE s0.st4;
+DROP FUNCTION testfunc();
+SELECT dbms_stats.clean_up_stats();
+
+-- No.16 error description. -- abnormal case.
+RESET SESSION AUTHORIZATION;
+CREATE TABLE s0.st4 (a int, b text);
+CREATE VIEW s0.vst4 AS select * FROM s0.st4;
+GRANT SELECT ON s0.vst4 TO regular_user;
+
+ALTER TABLE dbms_stats.relation_stats_locked OWNER TO regular_user;
+/* reconnection needed to flush cache */
+\c - regular_user
+
+EXPLAIN (COSTS OFF) SELECT * FROM s0.vst4 WHERE a = 1;
+
+\c - super_user
+ALTER TABLE dbms_stats.relation_stats_locked OWNER TO super_user;
+DROP TABLE s0.st4 CASCADE;
